/*

	Run Modeler Plugin v1.3
	Matt Gorner
	www.pixsim.co.uk

	History:

	1.0 (02.03.2009) Initial release

	1.1 Made the script search and run "MeshDataEdit" Classes of plugins
		Added code to check plugin type ("CommandSequence" or "MeshDataEdit")

	1.2 (20.07.2010) Added saving of search string across sessions

	1.3 (05.02.2011) Added support for LightWave 10, plus some small fixes

	- - -
	
	Collects and presents all the names of the plugins in the LWEXT*.cfg file alphabetically!  Also allows for searching.

	Useful if you have just installed a plugin and you want to test temporarily, but don't want to add a new button for it just yet.
	
	
	Thanks to Mike Green for his excellent LScript resource:
	http://www.mikegreen.name/Lscript/Lscript%20Index.html
	
	Without which I would have been very stuck!

*/

@version 2.2
@warnings
@script modeler
@name Run Modeler Plugin
@define DEBUG 0

// Global Variables ...
pluginlist;			// Master list of all plugin names
plugintype;			// Master list of whether plugin is CommandSequence or MeshDataEdit
pluginlistdisplay;	// Array used to display plugin names (rebuilt when filtered)
plugintypedisplay;	// Matching array of types (rebuilt when filtered)
searchstring = "";	// Search string used for "gad_search" ctlstring gadget
plugintorun;		// Index from ctrlistbox gadget that contains the selected plugin

// Declare the gadgets we need to gather data from across procedures
gad_search;
gad_plugin_list;

// User interface layout variables
req_width		= 525;
req_height		= 463;

layout_x		= 8;
layout_y		= 10;
layout_width	= 500;
layout_height	= 19;

slot_offset 	= 28;
spacer			= 5;
text_offset 	= 45;

UI				= FALSE;


// Main entry point ...
main
{

	searchstring = recall("rmp_searchstring", "");

	// Get config files path
	config = getdir(SETTINGSDIR);

	// Get system folder separator
	sep = getsep();

	// Build path to this scripts settings file
	settingsfile = config + sep + "run_plugin_prefs.cfg";

	// Does it exist (it won't if this plugin is run for the first time)
	if(fileexists(settingsfile))
	{
		// It exists, read in the path to the LWEXT*.cfg file
		settingsfileIO = File(settingsfile,"r") || error("An error occured opening settings file");
		configfile = settingsfileIO.read();
		settingsfileIO.close();
	}else{
		if( hostVersion().asInt() > 9 )
		{
			// It didn't exist, ask the user where it is
			configfile = getfile("Please select your LWEXT*.cfg or Extension Cache file (you only need to do this once) ...","*.*",config,1);
		}else
		{
			// It didn't exist, ask the user where it is
			configfile = getfile("Please select your LWEXT*.cfg file (you only need to do this once) ...","*.cfg",config,1);
		}

		// Did the user select a file?
		if(configfile)
		{
			// Yes, now store the path to that file into the scripts settings file
			settingsfile = config + sep + "run_plugin_prefs.cfg";
			settingsfileIO = File(settingsfile,"w") || error("An error occured opening settings file");
			settingsfileIO.writeln(configfile);
			settingsfileIO.close();
		}else
		{
			if( hostVersion().asInt() > 9 )
			{
				// No they didn't, tell them!
				error("You have not selected your LWEXT*.cfg or Extension Cache file");
			}else
			{
				// No they didn't, tell them!
				error("You have not selected your LWEXT*.cfg file");
			}
		}
	}

	// Open the LWEXT*.cfg file for reading ...
	configfileIO = File(configfile,"r") || error("Cannot open LWEXT*.cfg file");

	// Read first line (and remove spaces at start and end, just in case)
	line = configfileIO.read();
	line.pack();
	line.trunc();

	// Read through LWEXT*.cfg file until end ...
	while( !configfileIO.eof() )
	{
		// Does the line match what we are looking for?
		if(line == "Class \"CommandSequence\"" || line == "Class \"MeshDataEdit\"" || "Class \"MeshEditTool\"")
		{
			// Is it a CommandSequence Class type?
 			if(line == "Class \"CommandSequence\"")
			{
				// The name of the plugin we need is on the next line, so read that in (and remove spaces at start and end, just in case)
				line = configfileIO.read();
				line.pack();
				line.trunc();

				// Now extract just the plugin name ...
				name = strsub(line, size("Name \"") + 1, size(line) - size("Name \"") - 1);
				
				// Add name to the list and add CommandSequence "code" of 0 to plugintype array
				pluginlist += name;
				plugintype += 0;

			} else if(line == "Class \"MeshDataEdit\"") // Must be a MeshDataEdit Class type, but check anyway!
			{
				// The name of the plugin we need is on the next line, so read that in (and remove spaces at start and end, just in case)
				line = configfileIO.read();
				line.pack();
				line.trunc();

				// Now extract just the plugin name ...
				name = strsub(line, size("Name \"") + 1, size(line) - size("Name \"") - 1);
				
				// Add name to the list and add MeshDataEdit "code" of 1 to plugintype array
				pluginlist += name;
				plugintype += 1;
			}
		}

		// Read next line of LWEXT*.cfg file (and remove spaces at start and end, just in case)
		line = configfileIO.read();
		line.pack();
		line.trunc();
	}

	// All done, close the file
	configfileIO.close();

	// Copy master plugin list to the one we will display in the ctllistbox gadget (we need to keep a master copy untouched for searching)
	(pluginlistdisplay, plugintypedisplay) = sort_list(pluginlist, plugintype);

	// Initialise requester

	reqbegin("Run Plugin ...");
	reqsize(req_width, req_height);

	// Layout gadgets

	row_number	= 0;

	gad_search	= ctlstring("Search", searchstring);
	gad_clear	= ctlbutton("Clear",0,"button_callback");
	ctlposition(gad_search, layout_x, layout_y + (row_number * slot_offset) + (spacer * 2), layout_width - 74, layout_height, text_offset);
	ctlposition(gad_clear, layout_width - 62, layout_y + (row_number * slot_offset) + (spacer * 2), 70, layout_height);

	row_number ++ ;

	gad_plugin_list = ctllistbox("Plugin List",300,300,"lb_count","lb_name","lb_select");
	ctlposition(gad_plugin_list, layout_x + text_offset, layout_y + (row_number * slot_offset) + (row_number * spacer), layout_width - text_offset - 22, 370, text_offset);

	// Set callback procedure for clear search button
	ctlrefresh( gad_search, "refresh_list_gadget");

	UI = TRUE;
	
	refresh_list_gadget(0);

	// Show requester, quit if it fails
	return if !reqpost();

		// Grab user values we need
		plugintorun	   = getvalue(gad_plugin_list);
		searchstring   = getvalue(gad_search);

	// Close requester
	reqend();

	UI = FALSE;
	
	store("rmp_searchstring", searchstring);

	// If a plugin was selected from the list, run it
	if(plugintorun)
	{
		// Was a CommandSequence Class plugin?
		if(plugintypedisplay[ plugintorun[1] ] == 0)
			cmdseq(pluginlistdisplay[ plugintorun[1] ]);

		// Was is a MeshDataEdit Class plugin?
		if(plugintypedisplay[ plugintorun[1] ] == 1)
			meshedit(pluginlistdisplay[ plugintorun[1] ] );
	}
}






// ************************************************************************************************************************************************************
// Gadget Callback Funtions
// ************************************************************************************************************************************************************

// Keep track of how many items are in the ctllistbox gadget
lb_count
{
	return(pluginlistdisplay.count());
}

// The list of names that show up in the ctllistbox gadget
lb_name: index
{

	return( pluginlistdisplay[index] );
}

// Which one was selected?
lb_select: index
{
	return( index == 1 );
}

// Refresh the items shown in the ctllistbox gadget after searching
refresh_list_gadget: value
{
	// Get the search string (convert to lowercase to remove any case-sensitivity)
	if(UI)
		searchstring = strlower(getvalue(gad_search));

	temp;
	temp2;

	// Reset the display name list (and corresponding 'type' list)
	pluginlistdisplay 	= temp;
	plugintypedisplay	= temp2;

	// Loop through the master plugin items list
	for(loop = 1; loop < pluginlist.size() + 1; loop++)
	{
		// Get the plugin name from the pluginlist array (convert to lowercase to remove any case-sensitivity)
		stringtocheck = strlower(pluginlist[loop]);

		// Does it contain what we're looking for?
		if( stringtocheck.contains(searchstring))
		{
			// Yes, then add it name list array (and corresponding 'type' list)
			pluginlistdisplay += pluginlist[loop];
			plugintypedisplay += plugintype[loop];
		}
	}

	// Update the gadget
	if(UI)
		requpdate();
}

// Callback procedure for the clear search button
button_callback
{
	// Clear the search string text
	setvalue(gad_search,"");
}

// Simple bubble sort routine
sort_list: names, types
{
	for(x = 1; x < names.size(); x++)
	{
		for(y = 1; y < names.size(); y++)
		{
			s = parse(",", names[ y ]);
			t = parse(",", names[ y + 1 ]);

			if(s[1] > t[1])
			{
	        	temp			= names[y + 1];
	        	names[y + 1]	= names[y];
	        	names[y]		= temp;
	        	
 	        	temp2			= types[y + 1];
	        	types[y + 1]	= types[y];
	        	types[y]		= temp2;
			}

			s = parse(",", names[ y ]);
			t = parse(",", names[ y + 1 ]);
		}
	}
	return( names, types );
}

DBinfo: str
{
	if(DEBUG == 0)
		return;

	info(str);
}